@@ -10,21 +10,26 @@
 //! Start looking at [AudioProcessor], then have a look at [AudioBuffer] and [MidiEventHandler].
 use std::marker::PhantomData;
 
+pub use num::Float;
+
+pub use atomic_float::AtomicF32;
 pub use audio_buffer::{AudioBuffer, InterleavedAudioBuffer};
 pub use midi::{MidiEventHandler, MidiMessageLike};
 
+/// Atomic F32 implementation with `num` trait implementations
+pub mod atomic_float;
 /// Provides an abstraction for audio buffers that works for CPAL and VST layouts
 pub mod audio_buffer;
 /// Provides an abstraction for MIDI processing that works for stand-alone and VST events
 pub mod midi;
 
 /// Options provided to the audio-processor before calling `process`.
-#[derive(Clone, Copy)]
+#[derive(Clone, PartialEq, Debug, Copy)]
 pub struct AudioProcessorSettings {
     sample_rate: f32,
     input_channels: usize,
     output_channels: usize,
-    block_size: u32,
+    block_size: usize,
 }
 
 impl Default for AudioProcessorSettings {
@@ -38,7 +43,7 @@ impl AudioProcessorSettings {
         sample_rate: f32,
         input_channels: usize,
         output_channels: usize,
-        block_size: u32,
+        block_size: usize,
     ) -> Self {
         AudioProcessorSettings {
             sample_rate,
@@ -64,7 +69,7 @@ impl AudioProcessorSettings {
     }
 
     /// The number of samples which will be provided on each `process` call
-    pub fn block_size(&self) -> u32 {
+    pub fn block_size(&self) -> usize {
         self.block_size
     }
 }
@@ -82,7 +87,7 @@ impl AudioProcessorSettings {
         self.output_channels = output_channels;
     }
 
-    pub fn set_block_size(&mut self, block_size: u32) {
+    pub fn set_block_size(&mut self, block_size: usize) {
         self.block_size = block_size;
     }
 }
@@ -90,7 +95,7 @@ impl AudioProcessorSettings {
 /// Represents an audio processing node.
 ///
 /// Implementors should define the SampleType the node will work over. See some [examples here](https://github.com/yamadapc/augmented-audio/tree/master/crates/audio-processor-standalone/examples).
-pub trait AudioProcessor: Send + Sync {
+pub trait AudioProcessor {
     type SampleType;
 
     /// Prepare for playback based on current audio settings
@@ -103,10 +108,33 @@ pub trait AudioProcessor: Send + Sync {
     );
 }
 
+/// Auto-implemented object version of the audio-processor trait.
+///
+/// Given a known buffer-type, audio-processors can be made into objects using this type.
+pub trait ObjectAudioProcessor<BufferType> {
+    fn prepare_obj(&mut self, _settings: AudioProcessorSettings) {}
+    fn process_obj(&mut self, data: &mut BufferType);
+}
+
+impl<SampleType, BufferType, Processor> ObjectAudioProcessor<BufferType> for Processor
+where
+    SampleType: Float + Send,
+    BufferType: AudioBuffer<SampleType = SampleType>,
+    Processor: AudioProcessor<SampleType = SampleType>,
+{
+    fn prepare_obj(&mut self, settings: AudioProcessorSettings) {
+        <Processor as AudioProcessor>::prepare(self, settings);
+    }
+
+    fn process_obj(&mut self, data: &mut BufferType) {
+        <Processor as AudioProcessor>::process(self, data);
+    }
+}
+
 /// An audio-processor which doesn't do any work.
 pub struct NoopAudioProcessor<SampleType>(PhantomData<SampleType>);
 
-impl<SampleType: Send + Sync> AudioProcessor for NoopAudioProcessor<SampleType> {
+impl<SampleType: Send> AudioProcessor for NoopAudioProcessor<SampleType> {
     type SampleType = SampleType;
 
     fn process<BufferType: AudioBuffer<SampleType = Self::SampleType>>(
@@ -131,21 +159,15 @@ impl<SampleType> Default for SilenceAudioProcessor<SampleType> {
     }
 }
 
-impl<SampleType: num::Float + Send + Sync> AudioProcessor for SilenceAudioProcessor<SampleType> {
+impl<SampleType: Float + Send> AudioProcessor for SilenceAudioProcessor<SampleType> {
     type SampleType = SampleType;
 
     fn process<BufferType: AudioBuffer<SampleType = Self::SampleType>>(
         &mut self,
         output: &mut BufferType,
     ) {
-        for sample_index in 0..output.num_samples() {
-            for channel_index in 0..output.num_channels() {
-                output.set(
-                    channel_index,
-                    sample_index,
-                    <BufferType as AudioBuffer>::SampleType::zero(),
-                );
-            }
+        for sample in output.slice_mut() {
+            *sample = SampleType::zero();
         }
     }
 }
